""" Entry program for checking safety

Write your test cases!
"""

import sys, os

from parsita import Success,Failure

from mc308.data_structures.ts import TS, handshake
from mc308.data_structures.parse_ts import parse_ts_from_stream
from mc308.data_structures.repr_ts import ts_to_dot

from mc308.data_structures.formula import ParseBexp
from mc308.algorithms.omega_regular import check_persistence

def main():

    to_try = [ (["./ts/counter.ts"], ["pos","zero"]),
               (["./ts/counter_2.ts"], ["pos","zero"]),
               (["./ts/producer.ts",
                 "./ts/consumer.ts",
                 "./ts/sys.ts"], ["uninitialized","initialized"]),
              ]

    for (ts_files, persistence_properties) in to_try:

        ts = None 
        for ts_file in ts_files:
            print("Parsing ", ts_file)
            with open(ts_file, 'r') as f:
                other = parse_ts_from_stream(f).or_die()
                if ts is None:
                    ts = other
                else:
                    ts = handshake(ts, other, ts.actions.intersection(other.actions))
        assert not ts is None


        for persistence_str in persistence_properties:
            persistence = ParseBexp().parse(persistence_str)
            res, cex = check_persistence(ts, persistence)

            print("Checking persistence ", persistence_str, " for: ")
            print(ts_to_dot(ts))

            if res:
                print("TS |= Persistence(%s)" % str(persistence))
            else:
                print("TS does NOT satisfy Persistence(%s)!" % str(persistence))

                print("Found the run ", cex)

                is_run, trace = ts.simulate(cex)
                assert(is_run)
                print("With trace ", trace)
            print()


if __name__ == "__main__":
    main()


