""" Entry program for checking omega regular properties.

"""

import sys

from parsita import Success,Failure

from mc308.data_structures.ts import TS
from mc308.data_structures.nba import NBA
from mc308.data_structures.repr_nfa import nfa_to_dot
from mc308.data_structures.parse_ts import parse_ts_from_stream
from mc308.data_structures.repr_ts import ts_to_dot, ts_to_png

from mc308.algorithms.omega_regular import check_omega_regular

def main():
    with open("./ts/counter.ts", 'r') as stream:
        ts = parse_ts_from_stream(stream).or_die()


    # Write NBA for ! P:
    # accept language where F G ! zero

    alphabet = [frozenset(),
                frozenset(["pos"]),
                frozenset(["zero"]),
                frozenset(["pos","neg"])]
    empty,zero,pos,both = alphabet

    def get_non_zero(sfrom,sto):
        return [
            (sfrom,empty,sto),
            (sfrom,pos,sto),
            (sfrom,both,sto)]

    nba = NBA([0,1,2],
              alphabet,
              # TR
              [(0,zero,0)] +
              get_non_zero(0,1) +
              get_non_zero(1,1) +
              [(1,zero,2)] +
              [(2, l, 2) for l in alphabet],
              [0],
              [1])


    res, cex = check_omega_regular(ts, nba)

    if res:
        print("The safety property holds!")
    else:
        print("The safety property does not hold.")
        print("Here's a counterexample of length %d showing how TS violates the property:" % len(cex))

        for i in range(len(cex)):
            print("%3d: %s %s" % (i, cex[i], ts.labels[cex[i]]))


if __name__ == "__main__":
    main()


