"""
Algorithms to check omega-regular properties.

"""

from typing import Tuple, Optional, List, Set

from mc308.data_structures.ts import *
from mc308.data_structures.nba import *
from mc308.data_structures.formula import ParseBexp, PropFormula


from pysmt.shortcuts import FALSE, Or



def check_persistence(ts : TS,
                      f_persistence : PropFormula) -> Tuple[bool, Optional[List[States]]]:
    """
    Check the persistence property:

    'eventually, globally the trace does not visit any state in f_persistence'

    The function returns:
    - (True, None)    if TS |= Pers(f_persistence)
    - (False, cex)    otherwise

    cex represents an INFINITE PATH of TS that does not satisfy the persistence property.

    Since cex is infinite, we represent it as a finite sequence of states:

    s0 s1 ... s-(i-1) si ... si

    such that the INFINITE path that violates the persistence property is:

    s0 s1 ... s-(i-1) (si s-(s+1) ... s-(s+k))^(\omega)

    In practice:
      - s0 s1 ... s-(i-1) is a finite prefix (it may also be of zero length)
      - (si s-(s+1) ... s-(s+k))^(\omega)  is a path fragment that repeats 
      infinitely often
      - (si s-(s+1) ... s-(s+k))^(\omega) \not \models f_persistence

      This path is something called a lasso-path because of its shape.


    HINT: implement the double dfs algorithmm, similarly to what you did
    for a NBA (you probably can factor the same implementation, but for now
    you can replicate your code).
    """

    raise NotImplementedError()



def check_omega_regular(ts : TS, A : NBA) -> Tuple[bool, Optional[List[States]]]:

    """
    This function checks if Traces(ts) \cap L(A) = empty

    where ts is a transition system and A is a Nondeterministc Buchi Automaton (NBA).
    The intent of A is to represent the complement of a omega-regular
    property (i.e., L(A) = not P), so that:

    Traces(ts) \cap L(A) = empty IFF ts \models P

    The function returns:
    - (True, None)    if Traces(ts) \cap L(A) = empty
    - (False, cex)    otherwise

    cex represents an INFINITE PATH of TS (see the description in
    check_persistence

    HINT: to solve the problem, constructs the synchronous product of ts and A,
    and then check the persistent property:

    'eventually, globally the product does not visit any final state from A'

    """

    raise NotImplementedError()


