""" Non-Deterministic State Automaton (DFA)

A NFA is the tuple: <states, alphabet, tr, init, final>

A state is an integer number of a tuple of states (inductive definition)
The alphabet is a finite set of symbols
An atomic proposition (Prop) is a string
A transition is a tuple in State x Action x State

Example of use:

# Creates the states
s1 = (1,)
s2 = (2,)

# Creates a NFA
t = NFA(set([s1, s2]),
        set(["a","b"]),
        set([(s1,"a",s1),(s2,"b",s1)]),
        set([s1]),
        set([s2]))
"""

from __future__ import annotations
from collections.abc import Mapping, Iterator, Iterable
from typing import TypeAlias, Type, Tuple, Callable, Optional, NewType, TypeVar, Union, TypedDict
from copy import deepcopy

from mc308.data_structures.ts import States, Prop, Trans

## from mc308.data_structures.formula import PropFormula
# from pysmt.shortcuts import FALSE, TRUE, Symbol

from functools import reduce, partial
import itertools


T = TypeVar('T')
class NFA():

    def __init__(self : NFA,
                 states : frozenset[States],
                 alphabet : frozenset[T],
                 tr : frozenset[Trans],
                 init : frozenset[States],
                 final : frozenset[States]) -> None:
        self.states = set([a for a in states])
        self.alphabet = set([a for a in alphabet])
        self.tr = set([t for t in tr])
        self.init = set([i for i in init])
        self.final = set([i for i in final])


    def make_complete(self : NFA) -> None:
        """
        Add a sink state, if needed
        """



