import sys

from typing import Sequence
from functools import reduce

from parsita import (
    reg, lit, repsep, TextParsers
)
from parsita.state import (
    Input, Output, Result
)
from mc308.data_structures.ts import TS


class TsParser(TextParsers, whitespace=r'[\s]*'): # type: ignore
    """
    Parser for a transition system
    """
    state = reg(r'[1-9]+[0-9]|[0-9]') > (
        lambda a : int(a)
    )
    ts = lit('TS')
    states = lit('states')
    actions = lit('actions')
    tr = lit('tr')
    init = lit('init')
    ap = lit('ap')
    labels = lit('labels')

    identifier = reg(r'[a-zA-Z][a-zA-Z0-9]*')

    states_decl = states >> ':' >> '{' >> repsep(state, ',') << '}' << ';'

    actions_decl = actions >> ':' >> '{' >> repsep(identifier, ',') << '}' << ';'

    trans = '(' >> state & ',' >> identifier & ',' >> state << ')' > (
        lambda l : (l[0],l[1],l[2])
    )

    tr_decl = tr >> ':' >> '{' >> repsep(trans, ',') << '}' << ';'

    init_decl = init >> ':' >> '{' >> repsep(state, ',') << '}' << ';'

    ap_decl = ap >> ':' >> '{' >> repsep(identifier, ',') << '}' << ';'

    label_decl = state << ':' << '{' & repsep(identifier, ',') << '}' > (
        lambda res : {res[0] : frozenset(res[1])}
    )

    labels_decl = labels >> ':' >> '{' >> repsep(label_decl,',') << '}' << ';' > (
        lambda labels : reduce(lambda acc, x : (acc.update(x), acc)[1],
                               labels, {}
                               )
    )

    ts_decl = ts >> ':' >> '{' >> \
        states_decl & \
        actions_decl & \
        tr_decl & \
        init_decl & \
        ap_decl & \
        labels_decl  << '}' > (
            lambda decl : TS(set(decl[0]), # type: ignore
                             set(decl[1]), # type: ignore
                             set(decl[2]), # type: ignore
                             set(decl[3]), # type: ignore
                             set(decl[4]), # type: ignore
                             decl[5]       # type: ignore
                             )
       )

def parse_ts(source: Sequence[Input]) -> Result[Output]:
    return TsParser.ts_decl.parse(source)

def parse_ts_from_stream(stream) -> Result[Output]:
    return parse_ts("".join(stream.readlines()))
