""" Module for printing the NFA.

TODO: reduce code duplication with the printing of TS (here, copy and paste)

Uses the graphviz library to create a DOT file.
"""

import io
import typing

import graphviz

from mc308 import IS_IN_IPYTHON # safe to use in 
from mc308.data_structures.nfa import NFA
from mc308.data_structures.nba import NBA

class DotConfig:
    """
    Configuration for printing in dot format (now empty)
    """
    def __init__(self):
        pass

def nfa_to_dot(nfa : NFA or NBA, config : DotConfig = DotConfig()) -> graphviz.Graph:
    """ Convert a transition system to a graph in the dot format """

    dot = graphviz.Digraph(comment='NFA')

    # Create nodes
    for n in sorted(list(nfa.states)):
        if n in nfa.final:
            dot.node('%s' % str(n), shape='doublecircle')
        else:
            dot.node('%s' % str(n))

        # Add initial states
        if n in sorted(list(nfa.init)):
            invisible_node = '_fake_init_%s' % str(n)
            dot.node(invisible_node, style="invisible")
            dot.edge('%s' % invisible_node, '%s' % str(n))

    # Create the edges
    for a in sorted(list(nfa.tr)):
        dot.edge(str(a[0]), str(a[2]), str(a[1]))

    return dot



def ts_to_png(nfa : NFA or NBA,
              outstream : typing.BinaryIO,
              config : DotConfig = DotConfig()) -> None:
    dot = nfa_to_dot(nfa, config)
    png_bytes = dot._repr_image_png()
    outstream.write(png_bytes)


# def ts_to_tex_stream(ts : ts.TS, out : typing.TextIO) -> None:
#     """ Produces a TEX representation of the transition system

#     <S, A, ->, I, AP, L>
#     """

#     def _print_sep_list(out, l, init=r'\{', end=r'\}', sep=",", sort=True, to_str=str):
#         new_l = sorted(l) if sort else l
#         out.write(init)
#         r = ""
#         for a in new_l:
#             out.write("%s%s" % (r,to_str(a)))
#             r = sep
#         out.write(end)

#     # Start printing TS
#     out.write("$ \\langle")

#     # States
#     _print_sep_list(out, list(ts.states))
#     out.write(",")

#     # Actions
#     _print_sep_list(out, list(ts.actions))
#     out.write(",")

#     # Transitions
#     print_tr = lambda a : "(%s,%s,%s)" % (str(a[0]), str(a[1]), str(a[2]))
#     _print_sep_list(out, list(ts.tr), r'\{', r'\}', ",", True, print_tr)
#     out.write(",")

#     # Init
#     _print_sep_list(out, list(ts.init))
#     out.write(",")

#     # AP
#     _print_sep_list(out, list(ts.ap))
#     out.write(",")

#     # L
#     out.write(r'\{')
#     labels_list = []
#     sep = ""
#     for s in ts.states:
#         out.write("%s%s : " % (sep,str(s)))
#         _print_sep_list(out,
#                         list(ts.labels[s]) if s in ts.labels else [])
#         sep = ","
#     out.write(r'\}')

#     out.write(" \\rangle $")

# def ts_to_tex(ts : ts.TS) -> None:
#     """ Produces a TEX representation of the transition system
#     """
#     f = io.StringIO("")
#     ts_to_tex_stream(ts, f)
#     return f.getvalue()

# # Defined only when in the IPYTHON environment.
# if IS_IN_IPYTHON:
#     from IPython.display import Math
#     def as_math(ts: ts.TS) -> Math:
#         return Math(ts_to_tex(ts))

#     def show(ts: ts.TS):
#         return display(as_math(ts))


